#include <proto.h>

#include "gtcycle.h"
#include "gtlabel.h"
#include "gtgroup.h"
#include "gtbutton.h"
#include "gtmenu.h"
#include "gtsymbol.h"
#include "gtmenu.h"
#include "gtapp.h"
#include "gtpixmap.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"
#include "gtconfig.h"

/************************************************************************\
*
* GtCycle class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init cycle.
*
\************************************************************************/
GtCycle *GtMkCycle(char *n,GtManager *p,GtTag *tgs)
{
	GtCycle *obj;
	
	if(!(obj=new GtCycle()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtCycle object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init cycle.
*
\************************************************************************/
GtCycle *GtMkCycle(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtCycle *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkCycle(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty cycle.
*
\************************************************************************/
GtCycle::GtCycle(void)
{
	cmenu=NULL;
	cnames=NULL;
	cmax=0;
	cval=0;
	dir=GTD_DOWN;
}

/************************************************************************\
*
* Destroy cycle.
*
\************************************************************************/
void GtCycle::Destroy(void)
{
	if(clabs)
		free(clabs);
	GtCascade::Destroy();
}

/************************************************************************\
*
* Setup cycle.
*
\************************************************************************/
sbool GtCycle::Init(char *nm,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	GtGroup *g;
	GtButton *b;
	char **strs=NULL;
	char *str=NULL;
	GtPixmap *pix=NULL;
	ulong n;
	int mxs=1,lxs,lys;
	
	if(!GtCascade::Init(nm,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_LabelStrings:
				strs=(char **)t->d.ptr;
				break;
			case GT_LabelNames:
				cnames=(char **)t->d.ptr;
				break;
			case GT_Value:
				cval=t->d.val;
				break;
		}
	}
	if(ind)
		ind->SetV(GT_Symbol,GTST_CYCLE,GT_DONE);
	if(cnames)
	{
		for(cmax=0; cnames[cmax]; cmax++)
			;
		if(cval>=cmax)
			cval=cmax-1;
		if(cmax)
		{
			if(!(clabs=(GtLabel **)malloc(sizeof(GtLabel *)*cmax)))
				return(FALSE);
			if(!(cmenu=GtMkMenu("cyclemenu",this,GT_Mapped,FALSE,GT_DONE)))
				return(FALSE);
			menu=cmenu;
			if(!(g=GtMkGroup("cyclegrp",cmenu,
					GT_FrameType,GTF_OUT,
					GT_Orient,GTO_VERT,
					GT_DONE)))
				return(FALSE);
			mxs=1;
			for(n=0; n<cmax; n++)
			{
				if(!(b=GtMkButton("cyclebut",g,NULL)))
					return(FALSE);
				b->AddCB(GT_Activate,GtCycle::Cycle,this);
				b->SetUserData((GtUDPtr)(ulong)n);
				if(strs && strs[n])
				{
					if(!(clabs[n]=GtMkLabel(cnames[n],b,GT_String,strs[n],GT_DONE)))
						return(FALSE);
				}
				else
				{
					if(!(clabs[n]=GtMkLabel(cnames[n],b,NULL)))
						return(FALSE);
				}
				clabs[n]->GetMinSize(&lxs,&lys);
				mxs=GTMAX(mxs,lxs);
			}
		}
	}
	clab=GtMkLabel("cyclelab",this,
		GT_String,"",
		GT_MinXSize,mxs,
		GT_DONE);
	if(clabs)
	{
		if(cval>=cmax)
			cval=cmax-1;
		clabs[cval]->GetV(GT_String,&str,GT_Pixmap,&pix,GT_DONE);
		if(str)
			clab->SetV(GT_String,str,GT_DONE);
		if(pix)
			clab->SetV(GT_Pixmap,pix,GT_DONE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Set cycle attributes.
*
\************************************************************************/
sbool GtCycle::Set(GtTag *tgs)
{
	GtTag *t;
	GtGroup *g;
	GtButton *b;
	char **strs=NULL;
	char *str=NULL;
	GtPixmap *pix=NULL;
	sbool ch=FALSE,vch=FALSE;
	ulong n;
	int mxs,lxs,lys;
	
	if(!GtCascade::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_LabelStrings:
				strs=(char **)t->d.ptr;
				break;
			case GT_LabelNames:
				cnames=(char **)t->d.ptr;
				ch=TRUE;
				break;
			case GT_Value:
				cval=t->d.val;
				vch=TRUE;
				break;
		}
	}
	if(ch && cnames)
	{
		if(cmenu)
			GtDestroy(cmenu);
		cmenu=NULL;
		menu=NULL;
		for(cmax=0; cnames[cmax]; cmax++)
			;
		if(cval>=cmax)
			cval=cmax-1;
		if(cmax)
		{
			if(!(clabs=(GtLabel **)malloc(sizeof(GtLabel *)*cmax)))
				return(FALSE);
			if(!(cmenu=GtMkMenu("cyclemenu",this,GT_Mapped,FALSE,GT_DONE)))
				return(FALSE);
			if(!(g=GtMkGroup("cyclegrp",cmenu,
					GT_FrameType,GTF_OUT,
					GT_Orient,GTO_VERT,
					GT_DONE)))
				return(FALSE);
			mxs=1;
			for(n=0; n<cmax; n++)
			{
				if(!(b=GtMkButton("cyclebut",g,NULL)))
					return(FALSE);
				b->AddCB(GT_Activate,GtCycle::Cycle,this);
				b->SetUserData((GtUDPtr)n);
				if(strs && strs[n])
				{
					if(!(clabs[n]=GtMkLabel(cnames[n],b,GT_String,strs[n],GT_DONE)))
						return(FALSE);
				}
				else
				{
					if(!(clabs[n]=GtMkLabel(cnames[n],b,NULL)))
						return(FALSE);
				}
				clabs[n]->GetMinSize(&lxs,&lys);
				mxs=GTMAX(mxs,lxs);
			}
			clab->SetV(GT_MinXSize,mxs,GT_DONE);
		}
	}
	if((vch || ch) && clabs)
	{
		if(cval>=cmax)
			cval=cmax-1;
		clabs[cval]->GetV(GT_String,&str,GT_Pixmap,&pix,GT_DONE);
		if(str)
			clab->SetV(GT_String,str,GT_DONE);
		if(pix)
			clab->SetV(GT_Pixmap,pix,GT_DONE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Get cycle attributes.
*
\************************************************************************/
void GtCycle::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtCascade::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_LabelNames:
				*ptr=(long)cnames;
				break;
			case GT_Value:
				*ptr=cval;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtCycle::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtCycle")==0)
		return(TRUE);
	else
		return(GtCascade::IsSubClassOf(cn));
}

/************************************************************************\
*
* Activate cycle from internal callback.
*
\************************************************************************/
void GtCycle::NotifyA(ulong tag,void *arg)
{
	GtCycleCD cd;
	char *str=NULL;
	GtPixmap *pix=NULL;
	ulong nr;
	
	if(tag==GT_Activate)
	{
		nr=(ulong)arg;
		if(clabs && (nr<=cmax))
		{
			cval=nr;
			clabs[nr]->GetV(GT_String,&str,GT_Pixmap,&pix,GT_DONE);
			if(str)
				clab->SetV(GT_String,str,GT_DONE);
			else
				clab->SetV(GT_Pixmap,pix,GT_DONE);
			cd.nr=nr;
			act.CallCB(this,&cd);
		}
	}
	else
		GtCascade::NotifyA(tag,arg);
}

/************************************************************************\
*
* Mouse cycle events.
*
\************************************************************************/
sbool GtCycle::Mouse(XEvent *xe)
{
	int b,s;
	ubyte tmp;
	int xp,yp;
	
	if(!xe)
		return(FALSE);
	switch(xe->type)
	{
		case ButtonPress:
			b=xe->xbutton.button;
			if(flags & GTWF_DISABLED)
				break;
			if(!(bflags & GTBF_PRESS) && (b==GTDEF_MENUBUTTON))	
			{
				bflags|=(GTBF_ARMED|GTBF_PRESS);
				this->Refresh();
				this->Arm(b,xe->xbutton.state);
				return(TRUE);
			}
			if(!(bflags & GTBF_PRESS) && (b==Button1))
			{
				bflags|=(GTBF_ARMED|GTBF_PRESS);
				this->Refresh();
				return(TRUE);
			}
			break;
		case ButtonRelease:
			b=xe->xbutton.button;
			if((bflags & GTBF_PRESS) && (b==GTDEF_MENUBUTTON))
			{
				tmp=bflags & GTBF_ARMED;
				if(!(mshell && (b==Button1)))
					bflags&=~(GTBF_ARMED|GTBF_PRESS);
				this->Refresh();
				if(tmp)
				{
					this->Activate(b,xe->xbutton.state);
				}
				return(TRUE);
			}
			if((bflags & GTBF_PRESS) && (b==Button1))
			{
				tmp=bflags & GTBF_ARMED;
				bflags&=~(GTBF_ARMED|GTBF_PRESS);
				this->Refresh();
				cval++;
				if(cval>=cmax)
					cval=0;
				if(tmp)
					this->NotifyA(GT_Activate,(void *)cval);
			}
			break;
		case EnterNotify:
			if((flags & GTWF_DISABLED) || (xe->xcrossing.mode!=NotifyNormal))
				break;
			s=xe->xcrossing.state;
			if(mshell && (s & GTDEF_MENUBMASK))
				bflags|=GTBF_PRESS;
			if((bflags & GTBF_PRESS) && !(bflags & GTBF_ARMED) && (s & Button1Mask))
			{
				bflags|=GTBF_ARMED;
				this->Refresh();
				return(TRUE);
			}
			if((bflags & GTBF_PRESS) && !(bflags & GTBF_ARMED))
			{
				bflags|=GTBF_ARMED;
				this->Refresh();
				this->Arm(0,s);
				return(TRUE);
			}
			break;
		case LeaveNotify:
			if((xe->xcrossing.mode!=NotifyNormal) ||
				(xe->xcrossing.detail==NotifyInferior))
				break;
			s=xe->xcrossing.state;
			if((bflags & GTBF_ARMED) && (s & Button1Mask))
			{
				bflags&=~GTBF_ARMED;
				this->Refresh();
				return(TRUE);
			}
			xp=xe->xcrossing.x;
			yp=xe->xcrossing.y;
			if(((dir==GTD_DOWN) && (yp>=ysize)) ||
				((dir==GTD_RIGHT) && (xp>=xsize)))
				break;
			if(bflags & GTBF_ARMED)
			{
				bflags&=~GTBF_ARMED;
				this->Refresh();
				this->DisArm(0,s);
				return(TRUE);
			}
			break;
	}
	return(FALSE);
}

/************************************************************************\
*
* Activate cycle callback.
*
\************************************************************************/
int GtCycle::Cycle(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	ulong n;
	
	cd=cd;
	n=(ulong)((GtButton *)obj)->GetUserData();
	((GtCycle *)ud)->NotifyA(GT_Activate,(void *)n);
	return(GTCB_CONT);
}

